<?php
namespace WCProductsWizard;

/**
 * WCProductsWizard Cart Class
 *
 * @class Cart
 * @version 5.2.0
 */
class Cart
{
    /**
     * Session key variable
     * @var string
     */
    public static $sessionKey = 'woocommerce-products-wizard-cart';

    // <editor-fold desc="Get content">
    /**
     * Get cart from the session
     *
     * @param integer $postId
     * @param array $args
     *
     * @return array
     */
    public static function get($postId, $args = [])
    {
        do_action('wcpw_get_cart', $postId, $args);

        $defaults = [
            'includeSteps' => [],
            'excludeSteps' => []
        ];

        $args = array_replace($defaults, $args);

        if (!empty($args['includeSteps']) && !is_array($args['includeSteps'])) {
            $args['includeSteps'] = [(int) $args['includeSteps']];
        }

        if (!empty($args['excludeSteps']) && !is_array($args['excludeSteps'])) {
            $args['excludeSteps'] = [(int) $args['excludeSteps']];
        }

        $cart = [];
        $stepsIds = Settings::getStepsIds($postId);
        $storage = Storage::get(self::$sessionKey, $postId);

        if ($storage) {
            foreach (array_filter((array) $storage) as $key => $item) {
                $item = is_array($item) ? $item : (array) unserialize($item);

                if (isset($item['product_id'], $item['step_id']) && $item['product_id']) {
                    if ((!empty($args['includeSteps']) && !in_array((int) $item['step_id'], $args['includeSteps']))
                        || (!empty($args['excludeSteps']) && in_array((int) $item['step_id'], $args['excludeSteps']))
                    ) {
                        continue;
                    }

                    $productId = isset($item['variation_id']) && $item['variation_id']
                        ? $item['variation_id']
                        : $item['product_id'];

                    $item = apply_filters(
                        'woocommerce_get_cart_item_from_session',
                        array_merge($item, ['data' => wc_get_product($productId)]),
                        $item,
                        $key
                    );
                }

                if (!empty($item)) {
                    $cart[$key] = $item;
                }
            }
        }

        // clear cart from any empty values
        $cart = array_filter($cart);

        // sort cart items by steps
        $cartCopy = $cart;
        $cart = [];

        foreach ($stepsIds as $stepId) {
            foreach ($cartCopy as $key => $item) {
                if (!isset($item['step_id']) || $item['step_id'] != $stepId) {
                    continue;
                }

                $cart[$key] = $item;

                unset($cartCopy[$key]);
            }
        }

        return apply_filters('wcpw_cart', $cart, $postId, $args);
    }

    /**
     * Get products and variations IDs from the cart by the step ID
     *
     * @param integer $postId
     * @param array $args
     *
     * @return array
     */
    public static function getProductsAndVariationsIds($postId, $args = [])
    {
        $cart = self::get($postId, $args);
        $output = [];

        foreach ($cart as $cartItem) {
            if (!isset($cartItem['product_id'])) {
                continue;
            }

            $output[] = $cartItem['product_id'];

            if (isset($cartItem['variation_id']) && !empty($cartItem['variation_id'])) {
                $output[] = $cartItem['variation_id'];
            }
        }

        return apply_filters('wcpw_cart_products_and_variations_ids', $output, $postId, $args);
    }

    /**
     * Get cart by the step ID
     *
     * @param integer $postId
     * @param integer $stepId
     *
     * @return array
     */
    public static function getByStepId($postId, $stepId)
    {
        $cart = self::get($postId, ['includeSteps' => [$stepId]]);

        return apply_filters('wcpw_cart_by_step_id', $cart, $postId, $stepId);
    }

    /**
     * Return cart product data or null
     *
     * @param integer $postId
     * @param integer $productId
     * @param null|integer|string $stepId - for specific step only
     *
     * @return bool|null
     */
    public static function getProductById($postId, $productId, $stepId = null)
    {
        $cart = self::get($postId);
        $output = null;

        foreach ($cart as $cartItem) {
            if (isset($cartItem['product_id'], $cartItem['step_id'])
                && $cartItem['product_id'] == $productId && (!$stepId || $cartItem['step_id'] == $stepId)
            ) {
                $output = $cartItem;

                break;
            }
        }

        return apply_filters('wcpw_cart_product_by_id', $output, $postId, $productId, $stepId);
    }

    /**
     * Get product cart key by product ID
     *
     * @param integer $postId
     * @param integer $productId
     * @param null|integer|string $stepId - for specific step only
     *
     * @return bool|null
     */
    public static function getProductKeyById($postId, $productId, $stepId = null)
    {
        $cart = self::get($postId);

        foreach ($cart as $cartItemKey => $cartItem) {
            if (isset($cartItem['product_id'], $cartItem['step_id'])
                && $cartItem['product_id'] == $productId && (!$stepId || $cartItem['step_id'] == $stepId)
            ) {
                return $cartItemKey;
            }
        }

        return null;
    }

    /**
     * Check product is in the cart
     *
     * @param integer $postId
     * @param integer $productId
     * @param null|integer|string $stepId - for specific step only
     *
     * @return bool
     */
    public static function productIsset($postId, $productId, $stepId = null)
    {
        return (bool) self::getProductById($postId, $productId, $stepId);
    }
    // </editor-fold>

    // <editor-fold desc="Adding">
    /**
     * Add a product to the cart
     *
     * @param integer $postId - wizard ID
     * @param array $itemData - product data
     *
     * @return string
     *
     * @throws \Exception
     */
    public static function addProduct($postId, $itemData)
    {
        $cartItem = $itemData;

        // ensure we don't add a variation to the cart directly by variation ID
        $id = (isset($itemData['variation_id']) && !empty($itemData['variation_id']))
            ? (int) $itemData['variation_id']
            : (int) $itemData['product_id'];

        $variation = isset($itemData['variation']) ? $itemData['variation'] : [];
        $variationId = (isset($itemData['variation_id']) && !empty($itemData['variation_id']))
            ? $itemData['variation_id']
            : 0;

        $cartItem['data'] = !empty($cartItem['data']) ? $cartItem['data'] : [];
        $cartItem['data']['step_id'] = $cartItem['step_id'];

        // emulate post data passing
        if (!empty($cartItem['request'])) {
            $_POST = $cartItem['request'];
        }

        // load cart item data - might be added by other plugins
        $cartItemData = (array) apply_filters(
            'woocommerce_add_cart_item_data',
            $cartItem['data'],
            $cartItem['product_id'],
            $variationId,
            $cartItem['quantity']
        );

        // sanitize variations
        if (isset($variation) && is_array($variation)) {
            foreach ($variation as &$variationItem) {
                $variationItem = sanitize_text_field($variationItem);
            }
        }

        // generate key
        $cartItemKey = WC()->cart->generate_cart_id(
            $cartItem['product_id'],
            $variationId,
            $variation,
            $cartItemData
        );

        $cartItem = apply_filters(
            'woocommerce_add_cart_item',
            array_merge(
                $cartItemData,
                [
                    'key' => $cartItemKey,
                    'product_id' => $cartItem['product_id'],
                    'variation_id' => isset($cartItem['variation_id']) ? $cartItem['variation_id'] : '',
                    'variation' => isset($cartItem['variation']) ? $cartItem['variation'] : [],
                    'step_id' => $cartItem['step_id'],
                    'quantity' => (float) $cartItem['quantity'],
                    'request' => $cartItem['request'],
                    'sold_individually' => Settings::getStep($postId, $cartItem['step_id'], 'sold_individually'),
                    'data' => wc_get_product($id)
                ]
            ),
            $cartItemKey
        );

        $cartItem = apply_filters('wcpw_add_to_cart_item', $cartItem, $postId, self::$sessionKey);

        do_action('wcpw_before_add_to_cart', $postId, $cartItemKey, $cartItem);

        Storage::set(self::$sessionKey, $postId, $cartItem, $cartItemKey);

        do_action('wcpw_after_add_to_cart', $postId, $cartItemKey, $cartItem);

        return $cartItemKey;
    }
    // </editor-fold>

    // <editor-fold desc="Remove and clear">
    /**
     * Remove the product from the cart by the product id
     *
     * @param integer $postId
     * @param integer|string $productId
     * @param null|integer|string $stepId - for specific step only
     *
     * @return bool
     */
    public static function removeByProductId($postId, $productId, $stepId = null)
    {
        do_action('wcpw_before_remove_by_product_id', $postId, $productId, $stepId);

        $productKey = self::getProductKeyById($postId, $productId, $stepId);

        if (!$productKey) {
            return false;
        }

        self::removeByProductKey($postId, $productKey);

        do_action('wcpw_after_remove_by_product_id', $postId, $productId, $stepId);

        return true;
    }

    /**
     * Remove the product from the cart by the product cart key
     *
     * @param integer $postId
     * @param integer|string $key
     *
     * @return bool
     */
    public static function removeByProductKey($postId, $key)
    {
        do_action('wcpw_before_remove_by_product_key', $postId, $key);

        Storage::remove(self::$sessionKey, $postId, $key);

        do_action('wcpw_after_remove_by_product_key', $postId, $key);

        return true;
    }

    /**
     * Remove the products from the cart by step Id
     *
     * @param integer $postId
     * @param integer|string $stepId
     */
    public static function removeByStepId($postId, $stepId)
    {
        do_action('wcpw_before_remove_by_step_id', $postId, $stepId);

        $cart = self::get($postId);

        foreach ($cart as $key => $item) {
            if ($item['step_id'] != $stepId) {
                continue;
            }

            self::removeByProductKey($postId, $key);
        }

        do_action('wcpw_after_remove_by_step_id', $postId, $stepId);
    }

    /**
     * Truncate the cart
     *
     * @param integer $postId
     */
    public static function truncate($postId)
    {
        do_action('wcpw_before_truncate', $postId);

        Storage::remove(self::$sessionKey, $postId);

        do_action('wcpw_after_truncate', $postId);
    }
    // </editor-fold>

    // <editor-fold desc="Get price">
    /**
     * Get the total price of the cart
     *
     * @param integer $postId
     *
     * @return number
     */
    public static function getTotal($postId)
    {
        $output = 0;

        foreach (self::get($postId) as $cartItem) {
            if (!isset($cartItem['data']) || !$cartItem['data']) {
                continue;
            }

            $product = $cartItem['data'];

            if (!$product || !$product->exists() || $cartItem['quantity'] <= 0) {
                continue;
            }

            if (WC()->cart->display_prices_including_tax()) {
                $price = wc_get_price_including_tax($product);
            } else {
                $price = wc_get_price_excluding_tax($product);
            }

            $output += (float) ($price * $cartItem['quantity']);
        }

        return apply_filters('wcpw_cart_total', $output, $postId);
    }
    // </editor-fold>

    // <editor-fold desc="Utils">
    /**
     * Generate cart item ID
     *
     * @param array $cartItem
     *
     * @return string|null
     */
    public static function generateProductId($cartItem)
    {
        if (!isset($cartItem['product_id'], $cartItem['step_id'])) {
            return null;
        }

        return WC()->cart->generate_cart_id(
            $cartItem['product_id'],
            isset($cartItem['variation_id']) ? $cartItem['variation_id'] : '',
            isset($cartItem['variation']) ? $cartItem['variation'] : [],
            ['step_id' => $cartItem['step_id']]
        );
    }
    // </editor-fold>
}
