<?php
namespace WCProductsWizard;

/**
 * WCProductsWizard Core Class
 *
 * @class Core
 * @version 8.3.0
 */
class Core
{
    // <editor-fold desc="Properties">
    /**
     * Self instance variable
     * @var Core The single instance of the class
     */
    protected static $instance = null;

    /**
     * Current working wizard ID
     * @var int
     */
    public $activeId = null;

    /**
     * Current working wizard step ID
     * @var string
     */
    public $activeStepId = null;

    /**
     * Storage instance variable
     * @var Storage
     */
    public $storage = null;

    /**
     * Cart instance variable
     * @var Cart
     */
    public $cart = null;

    /**
     * Admin part instance variable
     * @var Admin
     */
    protected $admin = null;

    /**
     * Template class instance variable
     * @var Template
     */
    public $template = null;

    /**
     * Product class instance variable
     * @var Product
     */
    public $product = null;

    /**
     * Settings class instance variable
     * @var Settings
     */
    public $settings = null;

    /**
     * Form class instance variable
     * @var Form
     */
    public $form = null;
    // </editor-fold>

    // <editor-fold desc="Core">
    /** Class Constructor */
    public function __construct()
    {
        // include base slave classes
        $requiredClasses = [
            'L10N',
            'Utils',
            'Settings',
            'Storage',
            'Cart',
            'Template',
            'Form',
            'Product'
        ];

        foreach ($requiredClasses as $requiredClass) {
            if (!class_exists('\\WCProductsWizard\\' . $requiredClass)
                && file_exists(__DIR__ . DIRECTORY_SEPARATOR . $requiredClass . '.php')
            ) {
                include_once(__DIR__ . DIRECTORY_SEPARATOR . $requiredClass . '.php');
            }
        }

        // init classes
        self::$instance = $this;

        $this->settings = new Settings();
        $this->storage = new Storage();
        $this->cart = new Cart();
        $this->template = new Template();
        $this->product = new Product();
        $this->form = new Form();

        // actions
        add_action('wp_enqueue_scripts', [$this, 'enqueue']);
        add_action('woocommerce_init', [$this, 'wcInitHook'], 1);
        add_action('wcpw_before_output', [$this, 'beforeOutputAction']);
        add_action('wcpw_after_output', [$this, 'afterOutputAction']);
    }

    /**
     * Get single class instance
     *
     * @static
     * @return Core
     */
    public static function instance()
    {
        if (self::$instance === null) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    /** Handles then woocommerce plugin is loaded */
    public function wcInitHook()
    {
        // start WC session variable if needed
        if ($this->settings->getGlobal('store_session_in_db')) {
            if (method_exists(WC(), 'initialize_session')) {
                WC()->initialize_session();
            }

            if (method_exists(WC()->session, 'set_customer_session_cookie')) {
                WC()->session->set_customer_session_cookie(true);
            }
        }

        // if is admin page
        if (is_admin()) {
            if (!class_exists('\\WCProductsWizard\\Admin')) {
                include_once(__DIR__ . DIRECTORY_SEPARATOR . 'Admin.php');
            }

            $this->admin = new Admin();
        }
    }

    /**
     * Handles on output call
     *
     * @param array $args
     */
    public function beforeOutputAction($args)
    {
        if (isset($args['id']) && $args['id']) {
            $this->activeId = (int) $args['id'];
        }
    }

    /** Handles after output call */
    public function afterOutputAction()
    {
        $this->activeId = null;
        $this->activeStepId = null;
    }

    /** Styles and scripts enqueue */
    public function enqueue()
    {
        wp_enqueue_script('jquery');
        wp_enqueue_script(
            'woocommerce-products-wizard-scripts',
            WC_PRODUCTS_WIZARD_PLUGIN_URL . 'assets/front/js/scripts.min.js',
            ['jquery'],
            WC_PRODUCTS_WIZARD_VERSION,
            true
        );

        wp_enqueue_style(
            'woocommerce-products-wizard-full',
            WC_PRODUCTS_WIZARD_PLUGIN_URL . 'assets/front/css/app-full.min.css',
            [],
            WC_PRODUCTS_WIZARD_VERSION
        );

        // WooCommerce assets versions before 3.0.0
        if (get_option('woocommerce_enable_lightbox') === 'yes') {
            $suffix = defined('SCRIPT_DEBUG') && SCRIPT_DEBUG ? '' : '.min';
            $assetsPath = str_replace(['http:', 'https:'], '', \WC()->plugin_url()) . '/assets';

            wp_enqueue_script(
                'prettyPhoto',
                "$assetsPath/js/prettyPhoto/jquery.prettyPhoto$suffix.js",
                ['jquery'],
                '3.1.6',
                true
            );

            wp_enqueue_script(
                'prettyPhoto-init',
                "$assetsPath/js/prettyPhoto/jquery.prettyPhoto.init$suffix.js",
                ['prettyPhoto'],
                '3.1.6',
                true
            );

            wp_enqueue_style('woocommerce_prettyPhoto', "$assetsPath/css/prettyPhoto.css");
        }
    }
    // </editor-fold>
}
