<?php
namespace WCProductsWizard;

/**
 * WCProductsWizard Form Class
 *
 * @class Form
 * @version 5.3.0
 */
class Form
{
    //<editor-fold desc="Properties">
    /**
     * Active steps session keys variable
     * @var string
     */
    public static $activeStepsSessionKey = 'woocommerce-products-wizard-active-step';

    /**
     * Previous steps session keys variable
     * @var string
     */
    public static $previousStepsSessionKey = 'woocommerce-products-wizard-previous-step';

    /**
     * Ajax actions variable
     * @var array
     */
    public $ajaxActions = [];

    /**
     * Notices array
     * @var array
     */
    public $notices = [];
    //</editor-fold>

    // <editor-fold desc="Core">
    /** Class Constructor */
    public function __construct()
    {
        $ajaxActions = [
            'wcpwSubmit' => 'submitAjax',
            'wcpwAddToMainCart' => 'addToMainCartAjax',
            'wcpwGetStep' => 'getAjax',
            'wcpwSkipStep' => 'skipAjax',
            'wcpwSkipAll' => 'skipAllAjax',
            'wcpwReset' => 'resetAjax',
            'wcpwRemoveCartProduct' => 'removeCartProductAjax'
        ];

        $this->ajaxActions = apply_filters('wcpw_form_ajax_actions', $ajaxActions);

        // actions
        add_action('wp_loaded', [$this, 'requests']);

        // ajax actions
        foreach ($this->ajaxActions as $ajaxActionKey => $ajaxActionFunction) {
            add_action("wp_ajax_nopriv_{$ajaxActionKey}", [$this, $ajaxActionFunction]);
            add_action("wp_ajax_{$ajaxActionKey}", [$this, $ajaxActionFunction]);
        }
    }

    /** Add request actions */
    public function requests()
    {
        if (is_admin()) {
            return;
        }

        $request = $_REQUEST;

        // handle no-js forms actions
        if (isset($request['woocommerce-products-wizard'], $request['id'])) {
            // add all to main cart
            if (isset($request['add-to-main-cart'])) {
                try {
                    $this->addToMainCart($request);
                } catch (\Exception $exception) {
                    $this->addNotice(
                        $exception->getCode() ? $exception->getCode() : self::getActiveStepId($request['id']),
                        [
                            'view' => 'custom',
                            'message' => $exception->getMessage()
                        ]
                    );
                }
            }

            // submit form handler
            if (isset($request['submit'])) {
                try {
                    $this->submit($request);
                } catch (\Exception $exception) {
                    $this->addNotice(
                        $exception->getCode() ? $exception->getCode() : self::getActiveStepId($request['id']),
                        [
                            'view' => 'custom',
                            'message' => $exception->getMessage()
                        ]
                    );

                    return;
                }
            }

            // simple actions
            if (isset($request['reset'])) {
                self::reset($request);
            }

            if (isset($request['skip-step'])) {
                self::skip($request);
            }

            if (isset($request['skip-all'])) {
                self::skipAll($request);
            }

            if (isset($request['get-step'])) {
                $request['stepId'] = $request['get-step'];

                self::get($request);
            }
        }
    }
    // </editor-fold>

    // <editor-fold desc="Notices">
    /**
     * Add notice by type into a queue
     *
     * @param string $stepId
     * @param array $massageData
     */
    public function addNotice($stepId, $massageData)
    {
        $this->notices[$stepId][] = $massageData;
    }

    /**
     * Return the queue of notices
     *
     * @param string $stepId - try to get messages from one step by id or output all of messages
     *
     * @return array
     */
    public function getNotices($stepId = null)
    {
        if ($stepId) {
            // return step's messages array or nothing
            return isset($this->notices[$stepId]) ? $this->notices[$stepId] : [];
        } else {
            // return all steps messages
            return array_reduce($this->notices, 'array_merge', []);
        }
    }

    /**
     * Blend in cart quantities to check the rules
     *
     * @param integer $wizardId
     * @param array $qtyCheckArgs
     *
     * @return array
     */
    public static function getCartQuantitiesToCheck($wizardId, $qtyCheckArgs)
    {
        $cart = Cart::get($wizardId);
        $productsToAdd = (array) $qtyCheckArgs['productsToAdd'];
        $qtyCheckArgs['productsToAdd'] = [];

        // convert form input keys to hash keys
        foreach ($productsToAdd as $data) {
            $cartKey = Cart::generateProductId($data);
            $qtyCheckArgs['productsToAdd'][$cartKey] = $data;
        }

        foreach ($cart as $cartItem) {
            if (!isset($cartItem['product_id'])) {
                continue;
            }

            $cartKey = Cart::generateProductId($cartItem);

            // have no this product passed - take it from the cart
            if (!isset($qtyCheckArgs['productsToAdd'][$cartKey])) {
                $qtyCheckArgs['productsToAdd'][$cartKey] = $cartItem;
                $qtyCheckArgs['productsToAddChecked'][$cartItem['step_id']][] = $cartItem['product_id'];
            }
        }

        return $qtyCheckArgs;
    }
    // </editor-fold>

    // <editor-fold desc="Main actions">
    /**
     * Handles form submit
     *
     * @param array $args
     *
     * @return boolean
     *
     * @throws \Exception
     */
    public function submit($args)
    {
        $defaults = [
            'id' => null, // wizard ID
            'stepId' => null,
            'incrementActiveStep' => true,
            'dropNotCheckedProducts' => true,
            'productsToAdd' => [],
            'productsToAddChecked' => []
        ];

        $args = array_merge($defaults, $args);
        $notCheckedProductsIds = [];

        do_action('wcpw_before_submit_form', $args);

        // make it an array for sure
        $args['productsToAddChecked'] = $args['productsToAddChecked'] == '[]'
            ? []
            : (array) $args['productsToAddChecked'];

        $stepsIds = array_unique(array_filter(array_keys($args['productsToAddChecked'])));

        foreach ($stepsIds as $stepId) {
            if (!Settings::getStep($args['id'], $stepId, 'several_products')) {
                Cart::removeByStepId($args['id'], $stepId);
            }

            if ($args['dropNotCheckedProducts'] && !empty($notCheckedProductsIds)) {
                foreach ($notCheckedProductsIds as $stepId => $productsIds) {
                    foreach ($productsIds as $productId) {
                        Cart::removeByProductId($args['id'], $productId, $stepId);
                    }
                }
            }
        }

        if (is_array($args['productsToAdd']) && !empty($args['productsToAdd'])) {
            foreach ($args['productsToAdd'] as $data) {
                $defaultData = [
                    'product_id' => null,
                    'variation_id' => null,
                    'variation' => [],
                    'quantity' => 1,
                    'step_id' => null,
                    'data' => [],
                    'request' => []
                ];

                $data = array_replace($defaultData, $data);

                // if product isn't selected
                if (!($data['product_id'] && $data['step_id'] && $data['quantity'])
                    || !isset($args['productsToAddChecked'][$data['step_id']])
                    || !in_array($data['product_id'], $args['productsToAddChecked'][$data['step_id']])
                ) {
                    continue;
                }

                // find variation ID if necessary
                if (!empty($data['variation']) && !$data['variation_id']) {
                    $product = wc_get_product($data['product_id']);

                    if (!$product instanceof \WC_Product_Variable) {
                        continue;
                    }

                    $variations = $product->get_available_variations();

                    foreach ($variations as $variationKey => $variation) {
                        $attributesMet = 0;

                        foreach ($variation['attributes'] as $attribute => $value) {
                            if (isset($data['variation'][$attribute])
                                && ($data['variation'][$attribute] == $value || $value == '')
                            ) {
                                $attributesMet++;
                            }
                        }

                        if (count($data['variation']) == $attributesMet) {
                            $data['variation_id'] = $variation['variation_id'];
                        }
                    }
                }

                try {
                    if (Cart::productIsset($args['id'], $data['product_id'], $data['step_id'])) {
                        Cart::removeByProductId($args['id'], $data['product_id'], $data['step_id']);
                    }

                    Cart::addProduct($args['id'], apply_filters('wcpw_submit_form_item_data', $data, $args));
                } catch (\Exception $exception) {
                    throw new \Exception($exception->getMessage(), $data['step_id']);
                }
            }
        }

        // change active step
        if ($args['stepId']) {
            $this->setActiveStep($args['id'], $args['stepId']);
        } elseif (filter_var($args['incrementActiveStep'], FILTER_VALIDATE_BOOLEAN)) {
            $this->setActiveStep($args['id'], $this->getNextStepId($args['id']));
        }

        do_action('wcpw_after_submit_form', $args);

        return true;
    }

    /** Handles form submit via ajax */
    public function submitAjax()
    {
        // $post variable might be overwritten
        $postData = Utils::parseArrayOfJSONs($_POST);

        try {
            $this->submit($postData);
        } catch (\Exception $exception) {
            $this->addNotice(
                $exception->getCode() ? $exception->getCode() : self::getActiveStepId($postData['id']),
                [
                    'view' => 'custom',
                    'message' => $exception->getMessage()
                ]
            );

            Utils::sendJSON([
                'hasError' => true,
                'message' => $exception->getMessage(),
                'content' => Template::html('router', $postData, ['echo' => false])
            ]);
        }

        Utils::sendJSON(['content' => Template::html('router', $postData, ['echo' => false])]);
    }

    /**
     * Handles adding products to the cart
     *
     * @param array $args
     *
     * @throws \Exception
     */
    public function addToMainCart($args)
    {
        $defaults = ['incrementActiveStep' => false];
        $args = array_merge($defaults, $args);

        // submit step once again
        $this->submit($args);
        $cart = Cart::get($args['id']);
        $qtyCheckArgs = [
            'productsToAddChecked' => [],
            'productsToAdd' => []
        ];

        foreach ($cart as $product) {
            $qtyCheckArgs['productsToAddChecked'][$product['step_id']][] = $product['product_id'];
            $qtyCheckArgs['productsToAdd'][$product['step_id'] . '-' . $product['product_id']] = $product;
        }

        // send product to the main cart
        Product::addAllToMainCart($args['id']);
    }

    /** Handles adding products to the cart via ajax */
    public function addToMainCartAjax()
    {
        // $post variable might be overwritten
        $postData = Utils::parseArrayOfJSONs($_POST);

        try {
            $this->addToMainCart($postData);
        } catch (\Exception $exception) {
            $this->addNotice(
                $exception->getCode() ? $exception->getCode() : self::getActiveStepId($postData['id']),
                [
                    'view' => 'custom',
                    'message' => $exception->getMessage()
                ]
            );

            Utils::sendJSON([
                'hasError' => true,
                'message' => $exception->getMessage(),
                'content' => Template::html('router', $postData, ['echo' => false])
            ]);
        }

        Utils::sendJSON([
            'finalRedirectUrl' => Settings::getFinalRedirectUrl($postData['id']),
            'preventRedirect' => apply_filters('wcpw_prevent_final_redirect', false, $postData)
        ]);
    }

    /**
     * Get the form template
     *
     * @param array $args
     */
    public static function get($args)
    {
        $defaults = [
            'id' => null,
            'stepId' => null,
            'page' => 1
        ];

        $args = array_merge($defaults, $args);

        do_action('wcpw_before_get_form', $args);

        self::setActiveStep($args['id'], $args['stepId']);
        self::resetPreviousStepId($args['id']);

        do_action('wcpw_after_get_form', $args);
    }

    /** Get the form template via ajax */
    public function getAjax()
    {
        $postData = Utils::parseArrayOfJSONs($_POST);

        self::get($postData);

        Utils::sendJSON(['content' => Template::html('router', $postData, ['echo' => false])]);
    }

    /**
     * Handles form skipping
     *
     * @param array $args
     */
    public static function skip($args)
    {
        $defaults = ['id' => null];
        $args = array_merge($defaults, $args);

        do_action('wcpw_before_skip_form', $args);

        $activeStep = self::getActiveStepId($args['id']);

        Cart::removeByStepId($args['id'], $activeStep);
        self::setActiveStep($args['id'], self::getNextStepId($args['id']));

        do_action('wcpw_after_skip_form', $args);
    }

    /** Handles form skipping via ajax */
    public function skipAjax()
    {
        $postData = Utils::parseArrayOfJSONs($_POST);

        self::skip($postData);

        Utils::sendJSON(['content' => Template::html('router', $postData, ['echo' => false])]);
    }

    /**
     * Handles all steps skipping
     *
     * @param array $args
     */
    public static function skipAll($args)
    {
        $defaults = ['id' => null];
        $args = array_merge($defaults, $args);

        do_action('wcpw_before_skip_all', $args);

        $stepsIds = self::getStepsIds($args['id']);

        self::setPreviousStepId($args['id'], self::getActiveStepId($args['id']));
        self::setActiveStep($args['id'], end($stepsIds));

        do_action('wcpw_after_skip_all', $args);
    }

    /** Handles all steps skipping via ajax */
    public function skipAllAjax()
    {
        $postData = Utils::parseArrayOfJSONs($_POST);

        self::skipAll($postData);

        Utils::sendJSON(['content' => Template::html('router', $postData, ['echo' => false])]);
    }

    /**
     * Reset cart and set the form to the first step
     *
     * @param array $args
     */
    public static function reset($args)
    {
        $defaults = ['id' => null];
        $args = array_merge($defaults, $args);

        do_action('wcpw_before_reset_form', $args);

        $stepsIds = self::getStepsIds($args['id']);
        $cart = Cart::get($args['id']);

        foreach ($cart as $key => $item) {
            if (!in_array($item['step_id'], $stepsIds)) {
                continue;
            }

            Cart::removeByProductKey($args['id'], $key);
        }

        Storage::remove(self::$previousStepsSessionKey, $args['id']);
        self::setActiveStep($args['id'], reset($stepsIds));

        do_action('wcpw_after_reset_form', $args);
    }

    /** Reset cart and set the form to the first step via ajax */
    public function resetAjax()
    {
        $postData = Utils::parseArrayOfJSONs($_POST);

        // unset for sure because this leads to a wrong router view
        unset($postData['stepId']);

        self::reset($postData);

        Utils::sendJSON(['content' => Template::html('router', $postData, ['echo' => false])]);
    }
    // </editor-fold>

    // <editor-fold desc="Product actions">
    /**
     * Remove product from the cart
     *
     * @param array $args
     *
     * @throws \Exception
     */
    public function removeCartProduct($args)
    {
        $defaults = [
            'id' => null,
            'productCartKey' => null
        ];

        $args = array_merge($defaults, $args);
        $cart = Cart::get($args['id']);
        $product = isset($cart[$args['productCartKey']]) ? $cart[$args['productCartKey']] : null;
        $activeStepId = self::getActiveStepId($args['id']);

        if ($product && $product['step_id'] != $activeStepId) {
            // collect all other cart products from the same step to check minimum quantities rules
            $cart = Cart::getByStepId($args['id'], $product['step_id']);
            $qtyCheckArgs = [
                'id' => $args['id'],
                'productsToAdd' => [],
                'productsToAddChecked' => []
            ];

            foreach ($cart as $cartItem) {
                if ($cartItem['product_id'] == $product['product_id']) {
                    continue;
                }

                $qtyCheckArgs['productsToAddChecked'][$cartItem['step_id']][] = $cartItem['product_id'];
                $qtyCheckArgs['productsToAdd'][$cartItem['step_id'] . '-' . $cartItem['product_id']] = [
                    'product_id' => $cartItem['product_id'],
                    'step_id' => $cartItem['step_id'],
                    'quantity' => $cartItem['quantity']
                ];
            }
        }

        do_action('wcpw_before_remove_cart_product', $args);

        Cart::removeByProductKey($args['id'], $args['productCartKey']);
    }

    /** Remove item from the cart via ajax */
    public function removeCartProductAjax()
    {
        $postData = Utils::parseArrayOfJSONs($_POST);
        $defaults = ['id' => null];
        $args = array_merge($defaults, $postData);

        try {
            $this->removeCartProduct($args);
        } catch (\Exception $exception) {
            $this->addNotice(
                self::getActiveStepId($args['id']),
                [
                    'view' => 'custom',
                    'message' => $exception->getMessage()
                ]
            );

            Utils::sendJSON([
                'hasError' => true,
                'message' => $exception->getMessage(),
                'content' => Template::html('router', $postData, ['echo' => false])
            ]);
        }

        Utils::sendJSON(['content' => Template::html('router', $args, ['echo' => false])]);
    }
    // </editor-fold>

    // <editor-fold desc="Steps">
    /**
     * Get an array of the steps ids which are used in the wizard
     *
     * @param integer $postId
     *
     * @return array
     */
    public static function getStepsIds($postId)
    {
        // have no cache because of the dynamic workflow
        $output = self::getSteps($postId, ['idsOnly' => true]);

        return apply_filters('wcpw_steps_ids', $output, $postId);
    }

    /**
     * Get an array of the steps which used in the wizard
     *
     * @param integer $postId
     * @param array $args
     *
     * @return array
     */
    public static function getSteps($postId, $args = [])
    {
        $defaults = ['idsOnly' => false];
        $args = array_merge($defaults, $args);
        $output = [];
        $stepsIds = Settings::getStepsIds($postId);

        foreach ($stepsIds as $stepId) {
            if ($args['idsOnly']) {
                $output[] = $stepId;

                continue;
            }

            $output[$stepId] = self::getStep($postId, $stepId);
        }

        // add description tab
        if ($args['idsOnly']) {
            array_unshift($output, 'start');
        } else {
            array_unshift($output, self::getStep($postId, 'start'));
        }

        // add results tab
        if ($args['idsOnly']) {
            $output[] = 'result';
        } else {
            $output[] = self::getStep($postId, 'result');
        }

        return apply_filters('wcpw_steps', $output, $postId);
    }

    /**
     * Get step data
     *
     * @param integer $postId
     * @param integer $stepId
     *
     * @return array
     */
    public static function getStep($postId, $stepId)
    {
        static $stepsCache = [];

        if (isset($stepsCache[$postId], $stepsCache[$postId][$stepId])) {
            return apply_filters('wcpw_step', $stepsCache[$postId][$stepId], $postId, $stepId);
        }

        $output = [];

        if (is_numeric($stepId)) {
            $description = Settings::getStep($postId, $stepId, 'description');
            $title = Settings::getStep($postId, $stepId, 'title');
            $output = [
                'id' => $stepId,
                'name' => $title ? $title : $stepId,
                'categories' => Settings::getStep($postId, $stepId, 'categories'),
                'descriptionPosition' => Settings::getStep($postId, $stepId, 'description_position'),
                'description' => Settings::getStep($postId, $stepId, 'description_auto_tags')
                    ? apply_filters('the_content', $description)
                    : do_shortcode($description)
            ];
        } elseif ($stepId == 'start') {
            $output = [
                'id' => 'start',
                'name' => Settings::getPost($postId, 'description_tab_title'),
                'description' => apply_filters('the_content', get_post_field('post_content', $postId)),
                'descriptionPosition' => 'top'
            ];
        } elseif ($stepId == 'result') {
            $output = [
                'id' => 'result',
                'name' => Settings::getPost($postId, 'results_tab_title')
            ];
        }

        $stepsCache[$postId][$stepId] = $output;

        return apply_filters('wcpw_step', $output, $postId, $stepId);
    }

    /**
     * Get active wizard step id from the session variable
     *
     * @param integer $postId
     *
     * @return string
     */
    public static function getActiveStepId($postId)
    {
        $stepsIds = self::getStepsIds($postId);
        $output = Storage::get(self::$activeStepsSessionKey, $postId);

        if ($output && in_array($output, $stepsIds)) {
            Instance()->activeStepId = $output;

            return apply_filters('wcpw_active_step_id', $output, $postId);
        }

        $output = reset($stepsIds);
        Instance()->activeStepId = $output;

        return apply_filters('wcpw_active_step_id', $output, $postId);
    }

    /**
     * Get active wizard step from the session variable
     *
     * @param integer $postId
     *
     * @return array
     */
    public static function getActiveStep($postId)
    {
        $output = self::getStep($postId, self::getActiveStepId($postId));

        return apply_filters('wcpw_active_step', $output, $postId);
    }

    /**
     * Set active wizard step to the session variable
     *
     * @param integer $postId
     * @param integer|string $step
     */
    public static function setActiveStep($postId, $step)
    {
        Instance()->activeStepId = $step;

        Storage::set(self::$activeStepsSessionKey, $postId, $step);
    }

    /**
     * Get the next active wizard step from the session variable
     *
     * @param integer $postId
     *
     * @return string|null
     */
    public static function getNextStepId($postId)
    {
        $stepsIds = self::getStepsIds($postId);
        $activeStep = self::getActiveStepId($postId);
        $prevStep = false;

        foreach ($stepsIds as $stepId) {
            if ($prevStep == $activeStep) {
                return $stepId;
            }

            $prevStep = $stepId;
        }

        return null;
    }

    /**
     * Set the previous active wizard step id
     *
     * @param integer $postId
     * @param integer $value
     */
    public static function setPreviousStepId($postId, $value)
    {
        Storage::set(self::$previousStepsSessionKey, $postId, $value);
    }

    /**
     * Reset the previous active wizard step id
     *
     * @param integer $postId
     */
    public static function resetPreviousStepId($postId)
    {
        Storage::remove(self::$previousStepsSessionKey, $postId);
    }

    /**
     * Get the previous active wizard step id
     *
     * @param integer $postId
     *
     * @return string|null
     */
    public static function getPreviousStepId($postId)
    {
        $value = Storage::get(self::$previousStepsSessionKey, $postId);

        if ($value) {
            return $value;
        }

        $stepsIds = self::getStepsIds($postId);
        $activeStep = self::getActiveStepId($postId);
        $prevStep = false;

        foreach ($stepsIds as $stepId) {
            if ($stepId == $activeStep) {
                return $prevStep;
            }

            $prevStep = $stepId;
        }

        return null;
    }
    // </editor-fold>

    // <editor-fold desc="Navigation">
    /**
     * Check previous step existence
     *
     * @param integer $postId
     *
     * @return boolean
     */
    public static function canGoBack($postId)
    {
        $stepsIds = self::getStepsIds($postId);
        $activeStep = self::getActiveStepId($postId);

        return reset($stepsIds) != $activeStep;
    }

    /**
     * Check next step existence
     *
     * @param integer $postId
     *
     * @return boolean
     */
    public static function canGoForward($postId)
    {
        $stepsIds = self::getStepsIds($postId);
        $activeStep = self::getActiveStepId($postId);

        return end($stepsIds) != $activeStep;
    }

    /**
     * Get pagination items array
     *
     * @param array $args
     *
     * @return array
     */
    public static function getPaginationItems($args)
    {
        $output = [];
        $pages = [];
        $defaults = [
            'stepId' => null,
            'page' => 1,
            'productsQuery' => []
        ];

        $args = array_merge($defaults, $args);

        if (!$args['productsQuery'] || empty($args['productsQuery'])) {
            return [];
        }

        if (isset($_REQUEST['wcpwPage'])) {
            if (is_string($_REQUEST['wcpwPage'])) {
                parse_str($_REQUEST['wcpwPage'], $pages);
            } else {
                $pages = $_REQUEST['wcpwPage'];
            }
        }

        $paginationArgs = [
            'format' => '?wcpwPage[' . $args['stepId'] . ']=%#%',
            'base' => '%_%',
            'total' => $args['productsQuery']->max_num_pages,
            'current' => isset($pages[$args['stepId']]) ? (int) $pages[$args['stepId']] : $args['page'],
            'show_all' => false,
            'end_size' => 1,
            'mid_size' => 2,
            'prev_next' => true,
            'prev_text' => L10N::r('« Previous'),
            'next_text' => L10N::r('Next »'),
            'type' => 'array'
        ];

        $paginationArgs = apply_filters('wcpw_pagination_args', $paginationArgs, $args);

        $links = paginate_links($paginationArgs);

        foreach ((array) $links as $link) {
            // add custom classes
            $link = str_replace('page-numbers', 'page-numbers page-link', $link);

            // replace empty href
            $link = str_replace('href=""', 'href="?paged=1"', $link);
            $link = str_replace("href=''", 'href="?paged=1"', $link);

            preg_match_all('/<a[^>]+href=([\'"])(?<href>.+?)\1[^>]*>/i', $link, $result);

            if (!empty($result) && !empty($result['href'][0])) {
                $href = $result['href'][0];
                $linkParts = parse_url($href);

                parse_str($linkParts['query'], $linkPartsQuery);

                // add custom attributes
                $link = str_replace(
                    ' href=',
                    ' data-component="wcpw-form-pagination-link" data-step-id="' . $args['stepId']
                    . '" data-page="'
                    . (isset($linkPartsQuery['wcpwPage'], $linkPartsQuery['wcpwPage'][$args['stepId']])
                        ? $linkPartsQuery['wcpwPage'][$args['stepId']] : 1)
                    . '" href=',
                    $link
                );
            }

            $output[] = [
                'class' => strpos($link, 'current') !== false ? 'active' : '',
                'innerHtml' => $link
            ];
        }

        return apply_filters('wcpw_pagination_items', $output, $args);
    }

    /**
     * Return nav tabs items array
     *
     * @param integer $postId
     *
     * @return array
     */
    public static function getNavItems($postId)
    {
        static $navItemsCache = [];

        if (isset($navItemsCache[$postId])) {
            return apply_filters('wcpw_nav_items', $navItemsCache[$postId], $postId);
        }

        $output = self::getSteps($postId);
        $activeStepId = self::getActiveStepId($postId);
        $nextStepId = self::getNextStepId($postId);
        $activeNavItem = null;
        $previousNavItem = null;

        foreach ($output as &$step) {
            if ($activeStepId == $step['id']) {
                // active step
                $activeNavItem = $step['id'];
                $step['action'] = 'none';
                $step['state'] = 'active';
                $step['class'] = 'active';
            } elseif ($step['id'] == $nextStepId) {
                // next active step
                $step['action'] = 'get-step';
                $step['state'] = 'default';
                $step['class'] = 'next-active';
            } else {
                // other items
                $step['action'] ='get-step';
                $step['state'] = 'default';
                $step['class'] = $activeNavItem ? 'default' : 'past';
            }
        }

        $navItemsCache[$postId] = $output;

        return apply_filters('wcpw_nav_items', $output, $postId);
    }
    // </editor-fold>
}
