<?php
namespace WCProductsWizard;

/**
 * WCProductsWizard Product Class
 *
 * @class Product
 * @version 4.4.0
 */
class Product
{
    /**
     * Get, filter and return available product attributes and variables
     *
     * @param array $arguments
     *
     * @return array
     */
    public static function getVariationArguments($arguments)
    {
        $defaults = [
            'id' => null,
            'stepId' => null,
            'product' => false,
            'cart' => [],
            'otherStepsCartIds' => [],
            'defaultAttributes' => []
        ];

        $arguments = array_replace($defaults, $arguments);

        $productId = $arguments['product']->get_id();
        $variations = $arguments['product']->get_available_variations();
        $attributes = $arguments['product']->get_variation_attributes();
        $cartProduct = null;
        $attributesToRemove = [];
        $attributesToSave = [];
        $attributesOutput = [];

        foreach ($variations as &$variation) {
            // change image size
            if (!empty($variation['image_src'])) {
                $src = wp_get_attachment_image_src(get_post_thumbnail_id($variation['variation_id']), 'shop_catalog');
                $variation['image_src'] = $src[0];
            } elseif (!empty($variation['image']['src'])) {
                $src = wp_get_attachment_image_src(get_post_thumbnail_id($variation['variation_id']), 'shop_catalog');
                $variation['image']['src'] = is_array($src) && isset($src[0]) ? $src[0] : '';
            }
        }

        unset($variation);

        // clean attributes to remove from attributes to save
        foreach ($attributesToSave as $attributeToSaveKey => $attributeToSaveValue) {
            if (!isset($attributesToRemove[$attributeToSaveKey])) {
                continue;
            }

            $attributesToRemove[$attributeToSaveKey] = array_diff(
                $attributesToRemove[$attributeToSaveKey],
                $attributeToSaveValue
            );
        }

        // find and remove unmet product attributes
        foreach ($attributesToRemove as $attributeToRemoveItemKey => $attributeToRemoveItemValue) {
            foreach ($attributes as $attributeKey => $attributeValue) {
                if (urldecode(str_replace('attribute_', '', $attributeToRemoveItemKey))
                    != mb_strtolower($attributeKey)
                ) {
                    continue;
                }

                foreach ($attributeToRemoveItemValue as $attributeToRemoveItemValueItem) {
                    foreach ($attributeValue as $attributeItemValueItemKey => $attributeItemValueItemValue) {
                        if (urldecode($attributeToRemoveItemValueItem) != urldecode($attributeItemValueItemValue)) {
                            continue;
                        }

                        // unset product attribute
                        unset($attributes[$attributeKey][$attributeItemValueItemKey]);
                    }
                }
            }
        }

        // find this product in the cart
        foreach ($arguments['cart'] as $cartItem) {
            if (isset($cartItem['product_id'], $cartItem['step_id'])
                && $productId == $cartItem['product_id'] && $cartItem['step_id'] == $arguments['stepId']
            ) {
                $cartProduct = $cartItem;

                break;
            }
        }

        // get pure attributes array
        foreach ($attributes as $attributeKey => $attributeValue) {
            $selectedAttribute = '';

            // set active product if have in the cart
            if (isset($_REQUEST['attribute_' . sanitize_title($attributeKey)])) {
                $selectedAttribute = $_REQUEST['attribute_' . sanitize_title($attributeKey)];
            } elseif ($cartProduct && isset($cartProduct['variation']['attribute_' . sanitize_title($attributeKey)])) {
                $selectedAttribute = $cartProduct['variation']['attribute_' . sanitize_title($attributeKey)];
            } elseif (isset($arguments['defaultAttributes'][sanitize_title($attributeKey)])) {
                $selectedAttribute = $arguments['defaultAttributes'][sanitize_title($attributeKey)];
            }

            // Get terms if this is a taxonomy - ordered
            if (taxonomy_exists($attributeKey)) {
                switch (wc_attribute_orderby($attributeKey)) {
                    case 'name': {
                        $args = [
                            'orderby' => 'name',
                            'hide_empty' => false,
                            'menu_order' => false
                        ];

                        break;
                    }

                    case 'id': {
                        $args = [
                            'orderby' => 'id',
                            'order' => 'ASC',
                            'menu_order' => false,
                            'hide_empty' => false
                        ];

                        break;
                    }

                    case 'menu_order': {
                        $args = ['menu_order' => 'ASC', 'hide_empty' => false];
                        break;
                    }

                    default: {
                        $args = [];
                    }
                }

                $terms = get_terms($attributeKey, $args);

                foreach ($terms as $term) {
                    if (!in_array($term->slug, $attributeValue)) {
                        continue;
                    }

                    $selected = sanitize_title($selectedAttribute) == sanitize_title($term->slug);
                    $attributesOutput[$attributeKey][] = [
                        'id' => $term->term_id,
                        'name' => $term->name,
                        'value' => $term->slug,
                        'selected' => $selected,
                        'thumbnailId' => get_term_meta($term->term_id, '_wcpw_thumbnail_id', true)
                    ];
                }
            } else {
                foreach ($attributeValue as $option) {
                    $selected = sanitize_title($selectedAttribute) == sanitize_title($option);
                    $attributesOutput[$attributeKey][] = [
                        'name' => $option,
                        'value' => $option,
                        'selected' => $selected
                    ];
                }
            }
        }

        $output = [
            'variations' => $variations,
            'attributes' => $attributesOutput
        ];

        return apply_filters('wcpw_variation_arguments', $output, $arguments);
    }

    /**
     * Return arguments for the products form template part
     *
     * @param integer $postId
     * @param integer $stepId
     * @param array $args
     * @param array $productsIds
     *
     * @return array
     */
    public static function getRequestArgs($postId, $stepId, $args = [], $productsIds = [])
    {
        $defaults = ['page' => 1];
        $args = array_replace($defaults, $args);
        $productsPerPage = Settings::getStep($postId, $stepId, 'products_per_page');
        $orderBy = Settings::getStep($postId, $stepId, 'order_by');
        $activeProductsIds = Cart::getProductsAndVariationsIds($postId, ['includeSteps' => $stepId]);
        $otherStepsCartIds = Cart::getProductsAndVariationsIds($postId, ['excludeSteps' => $stepId]);
        $otherStepsCartIds[] = 1;

        // get products by filtered ids
        if (empty($productsIds)) {
            $productsIds = self::getStepProductsIds($postId, $stepId, $args);
        }

        // product request by current category
        $queryArgs = [
            'orderby' => $orderBy,
            'order' => Settings::getStep($postId, $stepId, 'order'),
            'post_type' => 'product',
            'post__in' => $productsIds,
            'posts_per_page' => -1,
            'numberposts' => -1,
            'paged' => $args['page']
        ];

        // args for price ordering
        if ($orderBy == 'price') {
            $queryArgs['orderby'] = 'meta_value_num';
            $queryArgs['meta_key'] = '_price';
        }

        // change products per page value
        if ($productsPerPage != 0) {
            $queryArgs['posts_per_page'] = $productsPerPage;
            $queryArgs['numberposts'] = $productsPerPage;
        }

        if (empty($activeProductsIds)) {
            // set the first product as active
            $productsQuery = get_posts(array_replace($queryArgs, ['numberposts' => 1]));
            $activeProductsIds[] = $productsQuery[0]->ID;
        }

        $output = [
            'id' => $postId,
            'queryArgs' => $queryArgs,
            'itemTemplate' => 'form/item/' . Settings::getStep($postId, $stepId, 'item_template'),
            'formId' => "wcpw-form-{$postId}",
            'stepId' => $stepId,
            'page' => $args['page'],
            'cart' => Cart::get($postId),
            'otherStepsCartIds' => $otherStepsCartIds,
            'activeProductsIds' => $activeProductsIds,
            'severalProducts' => Settings::getStep($postId, $stepId, 'several_products'),
            'descriptionSource' => Settings::getStep($postId, $stepId, 'item_description_source'),
            'soldIndividually' => Settings::getStep($postId, $stepId, 'sold_individually')
        ];

        return apply_filters('wcpw_products_request_args', $output, $productsIds, $args);
    }

    /**
     * Makes the products query considering all conditions
     *
     * @param integer $postId
     * @param integer $stepId
     * @param array $args
     *
     * @return string
     */
    public static function request($postId, $stepId, $args = [])
    {
        // there are no products
        if (empty(Settings::getStep($postId, $stepId, 'categories'))) {
            return '';
        }

        $productsIds = self::getStepProductsIds($postId, $stepId, $args);

        if (!empty($productsIds)) {
            $queryArgs = self::getRequestArgs($postId, $stepId, $args, $productsIds);
            $template = Settings::getStep($postId, $stepId, 'template');

            return Template::html("form/layouts/{$template}", $queryArgs);
        }

        return Template::html('messages/nothing-found', ['id' => $postId]);
    }

    /**
     * Add an array of the products to the main woocommerce cart
     *
     * @param integer $postId - wizard ID
     * @param array|null $cart - items to add array or cart content
     *
     * @throws \Exception
     */
    public static function addAllToMainCart($postId, $cart = null)
    {
        $stepsIds = Form::getStepsIds($postId);
        $productsAdded = [];
        $cart = $cart ? $cart : Cart::get($postId);
        $cart = apply_filters('wcpw_add_all_to_main_cart_items', $cart, $postId);

        do_action('wcpw_before_add_all_to_main_cart', $postId, $cart);

        // main work lower
        if (!empty($cart)) {
            foreach ($cart as $key => &$cartItem) {
                // should have a step ID and be not an excluded product/variation/step
                if (!isset($cartItem['step_id']) || !isset($cartItem['product_id'])) {
                    continue;
                }

                $productData = [
                    'productId' => $cartItem['product_id'],
                    'quantity' => $cartItem['quantity'],
                    'variationId' => isset($cartItem['variation_id']) ? $cartItem['variation_id'] : null,
                    'variation' => isset($cartItem['variation']) ? $cartItem['variation'] : [],
                    'data' => isset($cartItem['data']) && is_array($cartItem['data']) ? $cartItem['data'] : [],
                    'request' => isset($cartItem['request']) && is_array($cartItem['request'])
                        ? $cartItem['request']
                        : null
                ];

                $productData['data']['wcpw_id'] = $postId;
                $productData = apply_filters('wcpw_main_cart_product_data', $productData, $postId, $cartItem);
                $cartItemKey = self::addToMainCart($productData);

                if ($cartItemKey) {
                    $productsAdded[$cartItemKey] = $productData;
                }
            }
        }

        // truncate the cart
        Cart::truncate($postId);

        // reset active step to the first
        Form::setActiveStep($postId, reset($stepsIds));

        do_action('wcpw_after_add_all_to_main_cart', $postId, $cart, $productsAdded);
    }

    /**
     * Add a product to the main woocommerce cart
     *
     * @param array $args
     *
     * @throws \Exception
     *
     * @return string|bool
     */
    public static function addToMainCart($args)
    {
        $defaults = [
            'productId' => null,
            'quantity' => 1,
            'variationId' => null,
            'variation' => [],
            'data' => ['wcpw_id' => null],
            'request' => null
        ];

        $args = array_replace_recursive($defaults, $args);
        $cartQuantity = 0;

        do_action('wcpw_before_add_to_main_cart', $args);

        // get the same product's quantity from the main cart and remove it
        if (apply_filters('wcpw_merge_cart_quantity', false, $args)) {
            $cart = \WC()->cart->get_cart();

            foreach ($cart as $cartItemKey => $cartItem) {
                if ($cartItem['product_id'] != $args['productId']
                    || $cartItem['variation_id'] != $args['variationId']
                    || $cartItem['variation'] != $args['variation']
                ) {
                    continue;
                }

                $cartQuantity += (float) $cartItem['quantity'];

                \WC()->cart->remove_cart_item($cartItemKey);
            }
        }

        if ($args['request']) {
            // emulate post data passing
            $_POST = $args['request'];
        }

        return \WC()->cart->add_to_cart(
            $args['productId'],
            $args['quantity'] + $cartQuantity,
            $args['variationId'],
            $args['variation'],
            $args['data']
        );
    }

    /**
     * Prepare step products request query considering all conditions
     *
     * @param integer $wizardId
     * @param integer $stepId
     * @param array $args
     *
     * @return array
     */
    public static function getStepProductsIds($wizardId, $stepId, $args = [])
    {
        if (!$stepId || !is_numeric($stepId)) {
            return apply_filters('wcpw_step_products_ids', [], $wizardId, $stepId, $args);
        }

        $output = [];
        $categories = Settings::getStep($wizardId, $stepId, 'categories');

        // product request by current category
        $queryArgs = [
            'post_type' => 'product',
            'fields' => 'ids',
            'posts_per_page' => -1,
            'numberposts' => -1,
            'tax_query' => ['relation' => 'AND'],
            'meta_query' => ['relation' => 'AND'],
            'post__not_in' => [],
            'post__in' => []
        ];

        // query products by ids and order
        if (!empty($categories)) {
            $queryArgs['tax_query'][] = [
                'taxonomy' => 'product_cat',
                'field' => 'id',
                'terms' => $categories
            ];
        }

        // outer filters
        $queryArgs = apply_filters('wcpw_step_products_query_args', $queryArgs, $wizardId, $stepId, $args);

        // make a query
        $products = get_posts($queryArgs);

        foreach ($products as $productId) {
            // different types might be requested
            switch (get_post_type($productId)) {
                default:
                case 'product':
                    $product = new \WC_Product((int) $productId);
                    break;

                case 'product_variation':
                    $product = new \WC_Product_Variation((int) $productId);
            }

            $available = $product->is_visible() && $product->is_purchasable()
                && ($product->is_in_stock() || $product->backorders_allowed());

            if (apply_filters('wcpw_product_availability', $available, $productId, $wizardId, $stepId, $args)) {
                $output[] = (int) $productId;
            }
        }

        return apply_filters('wcpw_step_products_ids', $output, $wizardId, $stepId, $args);
    }
}
